/*
  RCTractor.h - Library for controlling RC Tractors.
  Created by Oisin O'Conchubhair, September 20, 2015.
  Released into the public domain.
*/

#include "Arduino.h"
#include "RCTractorControllerV1.h"
#include "RCTractorControllerV2MenuBackend.h"
#include <SPI.h>
#include "RCTractorControllernRF24L01.h"
#include "RCTractorControllerRF24.h"
#include <LiquidCrystal.h>

// Single radio pipe address for the 2 nodes to communicate.
const uint64_t pipe = 0xE8E8F0F0E1LL;

// Set up nRF24L01 radio on SPI bus plus pins 9 & 10
RF24 radio(49, 53);


// our LCD wiring
LiquidCrystal lcd(20, 22, 24, 26, 28, 30);

RCTractorControllerV1 RC_Library_TC;

/*******************************************************************/
/*               Variables for Voltage Measurements                */
/*******************************************************************/

// We're going to average the voltage readings
static const int numReadings = 10;
static int Controller_V_Readings[numReadings];      // the readings from the analog input
static float Vehicle_V_Readings[numReadings];      // the readings from the analog input
static int Controller_V_ReadIndex = 0;              // the index of the current reading
static int Vehicle_V_ReadIndex = 0;              // the index of the current reading
static int Controller_V_Total = 0;                  // the running total
static float Vehicle_V_Total = 0;                  // the running total
static float Vehicle_V_Average = 0;                // the average
static int Controller_V_Measure_Period = 0;
static int Vehicle_V_Measure_Period = 0;
static float Denominator;
static float Controller_V_Average;
static float Old_Controller_V_Average;
static float Old_Vehicle_V_Average;
static float Vehicle_Volt;
static long Vehicle_V;

/*******************************************************************/
/*                        General Variables                        */
/*******************************************************************/

static int Loop_Counter = 0;
static int Current_Light_Symbol=0;
static int Flash_Timer = 0;
static int Vehicle_No = 255;
static int New_Vehicle_No = 255;
static uint8_t command[11];
static uint8_t old_command[11];
static int IR_Heli_Active = 0;
static int TV_Active = 0;
static int Game_Active = 0;
static int Tool_Active = 0;
static int Vehicle_Active = 0;
static int Vehicle_Gear = 3;
static int Dashx = 180; // Dashboard Origin Coord X
static int Dashy = 150; // Dashboard Origin Coord Y
static int Controllerx = 210; // Dashboard Origin Coord X
static int Controllery = 25; // Dashboard Origin Coord Y
static int Throttle_Position = 0;
static int Lost_Packet_Count = 0;
static int Reception_Meter = 0;
static int Battery_Meter = 0;
static int EN_Mode = 5;
static int Control32_Active = 0;
static int Humanoid_Active = 0;
const char* Current_Vehicle_Name;
static int Com_Method = 0;
static unsigned long Send_Timer;


/*******************************************************************/
/*                      Menu Setup and Control                     */
/*******************************************************************/

static int Menu_Change = 0;
static int Menu_Level = 0;

	//beneath is list of menu items needed to build the menu
	MenuItem MainMenu1 = MenuItem("RC Farm");
		MenuItem Farm1 = MenuItem("Ford TW35");
		MenuItem Farm2 = MenuItem("John Deere 6920S");
		MenuItem Farm3 = MenuItem("John Deere 8360RT");
		MenuItem Farm4 = MenuItem("John Deere 9560R");
		MenuItem Farm5 = MenuItem("Massey 8680");
		MenuItem Farm6 = MenuItem("Fendt 936");
		MenuItem Farm7 = MenuItem("Claas Xerion 5000");
		MenuItem Farm8 = MenuItem("Claas Jaguar");
		MenuItem Farm9 = MenuItem("Ford 7810");
		MenuItem Farm10 = MenuItem("Tractor 10");

	MenuItem MainMenu2 = MenuItem("RC Haulage");
		MenuItem Haulage1 = MenuItem("Scania R620");
		MenuItem Haulage2 = MenuItem("MAN Dumper");
		MenuItem Haulage3 = MenuItem("Truck 3");
		MenuItem Haulage4 = MenuItem("Truck 4");
		MenuItem Haulage5 = MenuItem("Truck 5");
		MenuItem Haulage6 = MenuItem("Truck 6");
		MenuItem Haulage7 = MenuItem("Truck 7");
		MenuItem Haulage8 = MenuItem("Truck 8");
		MenuItem Haulage9 = MenuItem("Truck 9");
		MenuItem Haulage10 = MenuItem("Truck 10");

	MenuItem MainMenu3 = MenuItem("RC Construction");
		MenuItem Construction1 = MenuItem("Hitachi Zaxis 210");
		MenuItem Construction2 = MenuItem("Komatsu PC400LC");
		MenuItem Construction3 = MenuItem("Liebherr A900C");
		MenuItem Construction4 = MenuItem("JCB 550");
		MenuItem Construction5 = MenuItem("Construction 5");
		MenuItem Construction6 = MenuItem("Construction 6");
		MenuItem Construction7 = MenuItem("Construction 7");
		MenuItem Construction8 = MenuItem("Construction 8");
		MenuItem Construction9 = MenuItem("Construction 9");
		MenuItem Construction10 = MenuItem("Construction 10");

	MenuItem MainMenu4 = MenuItem("RC Trailers");
		MenuItem Trailer1 = MenuItem("Kane Silage Trailer");
		MenuItem Trailer2 = MenuItem("Trailer 2");
		MenuItem Trailer3 = MenuItem("Trailer 3");
		MenuItem Trailer4 = MenuItem("Trailer 4");
		MenuItem Trailer5 = MenuItem("Trailer 5");
		MenuItem Trailer6 = MenuItem("Trailer 6");
		MenuItem Trailer7 = MenuItem("Trailer 7");
		MenuItem Trailer8 = MenuItem("Trailer 8");
		MenuItem Trailer9 = MenuItem("Trailer 9");
		MenuItem Trailer10 = MenuItem("Trailer 10");


void menuUseEvent(MenuUseEvent used)
{
	if(used.item == Farm1){Vehicle_Active = 1; Current_Vehicle_Name = {Farm1.getName()}; Com_Method = 0; RC_Library_TC.Start_Vehicle();}
	if(used.item == Farm2){Vehicle_Active = 2; Current_Vehicle_Name = {Farm2.getName()}; Com_Method = 0; RC_Library_TC.Start_Vehicle();}
	if(used.item == Farm3){Vehicle_Active = 3; Current_Vehicle_Name = {Farm3.getName()}; Com_Method = 0; RC_Library_TC.Start_Vehicle();}
	if(used.item == Farm4){Vehicle_Active = 4; Current_Vehicle_Name = {Farm4.getName()}; Com_Method = 0; RC_Library_TC.Start_Vehicle();}
	if(used.item == Farm5){Vehicle_Active = 5; Current_Vehicle_Name = {Farm5.getName()}; Com_Method = 0; RC_Library_TC.Start_Vehicle();}
	if(used.item == Farm6){Vehicle_Active = 6; Current_Vehicle_Name = {Farm6.getName()}; Com_Method = 0; RC_Library_TC.Start_Vehicle();}
	if(used.item == Farm7){Vehicle_Active = 7; Current_Vehicle_Name = {Farm7.getName()}; Com_Method = 0; RC_Library_TC.Start_Vehicle();}
	if(used.item == Farm8){Vehicle_Active = 8; Current_Vehicle_Name = {Farm8.getName()}; Com_Method = 0; RC_Library_TC.Start_Vehicle();}
	if(used.item == Farm9){Vehicle_Active = 9; Current_Vehicle_Name = {Farm9.getName()}; Com_Method = 0; RC_Library_TC.Start_Vehicle();}
	if(used.item == Farm10){Vehicle_Active = 10; Current_Vehicle_Name = {Farm10.getName()}; Com_Method = 0; RC_Library_TC.Start_Vehicle();}

	if(used.item == Haulage1){Vehicle_Active = 11; Current_Vehicle_Name = {Haulage1.getName()}; Com_Method = 0; RC_Library_TC.Start_Vehicle();}
	if(used.item == Haulage2){Vehicle_Active = 12; Current_Vehicle_Name = {Haulage2.getName()}; Com_Method = 0; RC_Library_TC.Start_Vehicle();}
	if(used.item == Haulage3){Vehicle_Active = 13; Current_Vehicle_Name = {Haulage3.getName()}; Com_Method = 0; RC_Library_TC.Start_Vehicle();}
	if(used.item == Haulage4){Vehicle_Active = 14; Current_Vehicle_Name = {Haulage4.getName()}; Com_Method = 0; RC_Library_TC.Start_Vehicle();}
	if(used.item == Haulage5){Vehicle_Active = 15; Current_Vehicle_Name = {Haulage5.getName()}; Com_Method = 0; RC_Library_TC.Start_Vehicle();}
	if(used.item == Haulage6){Vehicle_Active = 16; Current_Vehicle_Name = {Haulage6.getName()}; Com_Method = 0; RC_Library_TC.Start_Vehicle();}
	if(used.item == Haulage7){Vehicle_Active = 17; Current_Vehicle_Name = {Haulage7.getName()}; Com_Method = 0; RC_Library_TC.Start_Vehicle();}
	if(used.item == Haulage8){Vehicle_Active = 18; Current_Vehicle_Name = {Haulage8.getName()}; Com_Method = 0; RC_Library_TC.Start_Vehicle();}
	if(used.item == Haulage9){Vehicle_Active = 19; Current_Vehicle_Name = {Haulage9.getName()}; Com_Method = 0; RC_Library_TC.Start_Vehicle();}
	if(used.item == Haulage10){Vehicle_Active = 20; Current_Vehicle_Name = {Haulage10.getName()}; Com_Method = 0; RC_Library_TC.Start_Vehicle();}

	if(used.item == Construction1){Vehicle_Active = 21; Current_Vehicle_Name = {Construction1.getName()}; Com_Method = 0; RC_Library_TC.Start_Hitachi();}
	if(used.item == Construction2){Vehicle_Active = 22; Current_Vehicle_Name = {Construction2.getName()}; Com_Method = 0; RC_Library_TC.Start_Vehicle();}
	if(used.item == Construction3){Vehicle_Active = 23; Current_Vehicle_Name = {Construction3.getName()}; Com_Method = 0; RC_Library_TC.Start_Vehicle();}
	if(used.item == Construction4){Vehicle_Active = 24; Current_Vehicle_Name = {Construction4.getName()}; Com_Method = 0; RC_Library_TC.Start_Vehicle();}
	if(used.item == Construction5){Vehicle_Active = 25; Current_Vehicle_Name = {Construction5.getName()}; Com_Method = 0; RC_Library_TC.Start_Vehicle();}
	if(used.item == Construction6){Vehicle_Active = 26; Current_Vehicle_Name = {Construction6.getName()}; Com_Method = 0; RC_Library_TC.Start_Vehicle();}
	if(used.item == Construction7){Vehicle_Active = 27; Current_Vehicle_Name = {Construction7.getName()}; Com_Method = 0; RC_Library_TC.Start_Vehicle();}
	if(used.item == Construction8){Vehicle_Active = 28; Current_Vehicle_Name = {Construction8.getName()}; Com_Method = 0; RC_Library_TC.Start_Vehicle();}
	if(used.item == Construction9){Vehicle_Active = 29; Current_Vehicle_Name = {Construction9.getName()}; Com_Method = 0; RC_Library_TC.Start_Vehicle();}
	if(used.item == Construction10){Vehicle_Active = 30; Current_Vehicle_Name = {Construction10.getName()}; Com_Method = 0; RC_Library_TC.Start_Vehicle();}
}


/*
	This is an important function
	Here we get a notification whenever the user changes the menu
	That is, when the menu is navigated
*/
void menuChangeEvent(MenuChangeEvent changed)
{
	// Main Menu
	if(changed.to.getName() == MainMenu1.getName()){Menu_Level = 0;lcd.clear();lcd.print(MainMenu1.getName());}
	if(changed.to.getName() == MainMenu2.getName()){Menu_Level = 0;lcd.clear();lcd.print(MainMenu2.getName());}
	if(changed.to.getName() == MainMenu3.getName()){Menu_Level = 0;lcd.clear();lcd.print(MainMenu3.getName());}
	if(changed.to.getName() == MainMenu4.getName()){Menu_Level = 0;lcd.clear();lcd.print(MainMenu4.getName());}

	//Sub Menu 1
	if(changed.to.getName() == Farm1.getName()){Menu_Level = 1;lcd.clear();lcd.print(Farm1.getName());}
	if(changed.to.getName() == Farm2.getName()){Menu_Level = 1;lcd.clear();lcd.print(Farm2.getName());}
	if(changed.to.getName() == Farm3.getName()){Menu_Level = 1;lcd.clear();lcd.print(Farm3.getName());}
	if(changed.to.getName() == Farm4.getName()){Menu_Level = 1;lcd.clear();lcd.print(Farm4.getName());}
	if(changed.to.getName() == Farm5.getName()){Menu_Level = 1;lcd.clear();lcd.print(Farm5.getName());}
	if(changed.to.getName() == Farm6.getName()){Menu_Level = 1;lcd.clear();lcd.print(Farm6.getName());}
	if(changed.to.getName() == Farm7.getName()){Menu_Level = 1;lcd.clear();lcd.print(Farm7.getName());}
	if(changed.to.getName() == Farm8.getName()){Menu_Level = 1;lcd.clear();lcd.print(Farm8.getName());}
	if(changed.to.getName() == Farm9.getName()){Menu_Level = 1;lcd.clear();lcd.print(Farm9.getName());}
	if(changed.to.getName() == Farm10.getName()){Menu_Level = 1;lcd.clear();lcd.print(Farm10.getName());}

	//Sub Menu 2
	if(changed.to.getName() == Haulage1.getName()){Menu_Level = 1;lcd.clear();lcd.print(Haulage1.getName());}
	else if(changed.to.getName() == Haulage2.getName()){Menu_Level = 1;lcd.clear();lcd.print(Haulage2.getName());}
	else if(changed.to.getName() == Haulage3.getName()){Menu_Level = 1;lcd.clear();lcd.print(Haulage3.getName());}
	else if(changed.to.getName() == Haulage4.getName()){Menu_Level = 1;lcd.clear();lcd.print(Haulage4.getName());}
	else if(changed.to.getName() == Haulage5.getName()){Menu_Level = 1;lcd.clear();lcd.print(Haulage5.getName());}
	else if(changed.to.getName() == Haulage6.getName()){Menu_Level = 1;lcd.clear();lcd.print(Haulage6.getName());}
	else if(changed.to.getName() == Haulage7.getName()){Menu_Level = 1;lcd.clear();lcd.print(Haulage7.getName());}
	else if(changed.to.getName() == Haulage8.getName()){Menu_Level = 1;lcd.clear();lcd.print(Haulage8.getName());}
	else if(changed.to.getName() == Haulage9.getName()){Menu_Level = 1;lcd.clear();lcd.print(Haulage9.getName());}
	else if(changed.to.getName() == Haulage10.getName()){Menu_Level = 1;lcd.clear();lcd.print(Haulage10.getName());}

	//Sub Menu 3
	if(changed.to.getName() == Construction1.getName()){Menu_Level = 1;lcd.clear();lcd.print(Construction1.getName());}
	else if(changed.to.getName() == Construction2.getName()){Menu_Level = 1;lcd.clear();lcd.print(Construction2.getName());}
	else if(changed.to.getName() == Construction3.getName()){Menu_Level = 1;lcd.clear();lcd.print(Construction3.getName());}
	else if(changed.to.getName() == Construction4.getName()){Menu_Level = 1;lcd.clear();lcd.print(Construction4.getName());}
	else if(changed.to.getName() == Construction5.getName()){Menu_Level = 1;lcd.clear();lcd.print(Construction5.getName());}
	else if(changed.to.getName() == Construction6.getName()){Menu_Level = 1;lcd.clear();lcd.print(Construction6.getName());}
	else if(changed.to.getName() == Construction7.getName()){Menu_Level = 1;lcd.clear();lcd.print(Construction7.getName());}
	else if(changed.to.getName() == Construction8.getName()){Menu_Level = 1;lcd.clear();lcd.print(Construction8.getName());}
	else if(changed.to.getName() == Construction9.getName()){Menu_Level = 1;lcd.clear();lcd.print(Construction9.getName());}
	else if(changed.to.getName() == Construction10.getName()){Menu_Level = 1;lcd.clear();lcd.print(Construction10.getName());}

	//Sub Menu 4
	if(changed.to.getName() == Trailer1.getName()){Menu_Level = 1;lcd.clear();lcd.print(Trailer1.getName());}
	else if(changed.to.getName() == Trailer2.getName()){Menu_Level = 1;lcd.clear();lcd.print(Trailer2.getName());}
	else if(changed.to.getName() == Trailer3.getName()){Menu_Level = 1;lcd.clear();lcd.print(Trailer3.getName());}
	else if(changed.to.getName() == Trailer4.getName()){Menu_Level = 1;lcd.clear();lcd.print(Trailer4.getName());}
	else if(changed.to.getName() == Trailer5.getName()){Menu_Level = 1;lcd.clear();lcd.print(Trailer5.getName());}
	else if(changed.to.getName() == Trailer6.getName()){Menu_Level = 1;lcd.clear();lcd.print(Trailer6.getName());}
	else if(changed.to.getName() == Trailer7.getName()){Menu_Level = 1;lcd.clear();lcd.print(Trailer7.getName());}
	else if(changed.to.getName() == Trailer8.getName()){Menu_Level = 1;lcd.clear();lcd.print(Trailer8.getName());}
	else if(changed.to.getName() == Trailer9.getName()){Menu_Level = 1;lcd.clear();lcd.print(Trailer9.getName());}
	else if(changed.to.getName() == Trailer10.getName()){Menu_Level = 1;lcd.clear();lcd.print(Trailer10.getName());}

}


//this controls the menu backend and the event generation
MenuBackend menu = MenuBackend(menuUseEvent,menuChangeEvent);





/*******************************************************************/
/*          RC Farm and Construction Vehicle Functions             */
/*******************************************************************/

RCTractorControllerV1::RCTractorControllerV1()
{
	_JX_L = A14;
	_JY_L = A12;
	_S_Left_Joy = 36;
	_JX_R = A15;
	_JY_R = A13;
	_S_Right_Joy = 25;

	_POT1 = A3;
	_POT2 = A5;
	_POT3 = A7;
	_POT4 = A9;

	_VQS_V1 = 12;
	_VQS_V2 = 10;
	_VQS_V3 = 8;
	_VQS_V4 = 6;
	_VQS_V5 = 4;
	_VQS_V6 = 2;


	_S_DB = 34;
	_S_FB = 35;
	_S_WL = 32;

	_S_F1 = 21;
	_S_F2 = 23;
	_S_F3 = 17;

	_S_4 = 33;
	_S_8 = 19;

	_S_Up = 43;
	_S_Down = 42;
	_S_Select = 29;
	_S_Back = 27;

	_DIP1 = 46;
	_DIP2 = 45;
	_DIP3 = 44;
	_DIP4 = 43;
	_DIP5 = 42;

	_IR_LED = 47;

	_LED_Data = 0;
	_FUNC_Data = 0;
	_Vehicle_No = 0;

	pinMode(_S_Left_Joy, INPUT_PULLUP);
	pinMode(_S_Right_Joy, INPUT_PULLUP);
	pinMode(_VQS_V1, INPUT_PULLUP);
	pinMode(_VQS_V2, INPUT_PULLUP);
	pinMode(_VQS_V3, INPUT_PULLUP);
	pinMode(_VQS_V4, INPUT_PULLUP);
	pinMode(_VQS_V5, INPUT_PULLUP);
	pinMode(_VQS_V6, INPUT_PULLUP);
	pinMode(_VQS_V7, INPUT_PULLUP);
	pinMode(_VQS_V8, INPUT_PULLUP);
	pinMode(_VQS_V9, INPUT_PULLUP);
	pinMode(_VQS_V10, INPUT_PULLUP);
	pinMode(_S_DB, INPUT_PULLUP);
	pinMode(_S_FB, INPUT_PULLUP);
	pinMode(_S_WL, INPUT_PULLUP);
	pinMode(_S_F1, INPUT_PULLUP);
	pinMode(_S_F2, INPUT_PULLUP);
	pinMode(_S_F3, INPUT_PULLUP);
	pinMode(_S_EN1, INPUT_PULLUP);
	pinMode(_S_EN2, INPUT_PULLUP);
	pinMode(_S_Up, INPUT_PULLUP);
	pinMode(_S_Down, INPUT_PULLUP);
	pinMode(_S_Select, INPUT_PULLUP);
	pinMode(_S_Back, INPUT_PULLUP);
	pinMode(_DIP1, INPUT_PULLUP);
	pinMode(_DIP2, INPUT_PULLUP);
	pinMode(_DIP3, INPUT_PULLUP);
	pinMode(_DIP4, INPUT_PULLUP);
	pinMode(_DIP5, INPUT_PULLUP);

	pinMode(_IR_LED, OUTPUT);

	pinMode(31, OUTPUT);
}

void RCTractorControllerV1::Setup(){



  digitalWrite(31,HIGH);

	  Serial.begin(9600);

	radio.begin();

	// RF24_PA_MIN, RF24_PA_LOW, RF24_PA_HIGH and RF24_PA_MAX
	radio.setPALevel(RF24_PA_LOW);

	// RF24_250KBPS for 250kbs, RF24_1MBPS for 1Mbps, or RF24_2MBPS for 2Mbps
	//radio.setDataRate(RF24_250KBPS);

	// We will be using the Ack Payload feature, so please enable it
	radio.enableAckPayload();

	radio.openWritingPipe(pipe);

	// Denomiator used in voltage measurements
	Denominator = (float)1000/(3300 + 1000);

	// Initialize all the measurement readings to 0:
	for (int thisReading = 0; thisReading < numReadings; thisReading++) {
		Controller_V_Readings[thisReading] = 0;
		Vehicle_V_Readings[thisReading] = 0;
	}

	// Menu Setup
	//add the file menu to the menu root
	menu.getRoot().add(MainMenu1);
	//setup the settings menu item
	MainMenu1.addRight(Farm1);
	//we want looping both up and down
		Farm1.addBefore(Farm10);
		Farm1.addAfter(Farm2);
		Farm2.addAfter(Farm3);
		Farm3.addAfter(Farm4);
		Farm4.addAfter(Farm5);
		Farm5.addAfter(Farm6);
		Farm6.addAfter(Farm7);
		Farm7.addAfter(Farm8);
		Farm8.addAfter(Farm9);
		Farm9.addAfter(Farm10);
		Farm10.addAfter(Farm1);

		//we want a left movement to pint to settings from anywhere
		Farm1.addLeft(MainMenu1);
		Farm2.addLeft(MainMenu1);
		Farm3.addLeft(MainMenu1);
		Farm4.addLeft(MainMenu1);
		Farm5.addLeft(MainMenu1);
		Farm6.addLeft(MainMenu1);
		Farm7.addLeft(MainMenu1);
		Farm8.addLeft(MainMenu1);
		Farm9.addLeft(MainMenu1);
		Farm10.addLeft(MainMenu1);

	MainMenu1.addBefore(MainMenu4);
	MainMenu1.addAfter(MainMenu2);

	MainMenu2.addRight(Haulage1);
	//we want looping both up and down
		Haulage1.addBefore(Haulage10);
		Haulage1.addAfter(Haulage2);
		Haulage2.addAfter(Haulage3);
		Haulage3.addAfter(Haulage4);
		Haulage4.addAfter(Haulage5);
		Haulage5.addAfter(Haulage6);
		Haulage6.addAfter(Haulage7);
		Haulage7.addAfter(Haulage8);
		Haulage8.addAfter(Haulage9);
		Haulage9.addAfter(Haulage10);
		Haulage10.addAfter(Haulage1);

		//we want a left movement to pint to settings from anywhere
		Haulage1.addLeft(MainMenu2);
		Haulage2.addLeft(MainMenu2);
		Haulage3.addLeft(MainMenu2);
		Haulage4.addLeft(MainMenu2);
		Haulage5.addLeft(MainMenu2);
		Haulage6.addLeft(MainMenu2);
		Haulage7.addLeft(MainMenu2);
		Haulage8.addLeft(MainMenu2);
		Haulage9.addLeft(MainMenu2);
		Haulage10.addLeft(MainMenu2);

	MainMenu2.addAfter(MainMenu3);

	MainMenu3.addRight(Construction1);
	//we want looping both up and down
		Construction1.addBefore(Construction10);
		Construction1.addAfter(Construction2);
		Construction2.addAfter(Construction3);
		Construction3.addAfter(Construction4);
		Construction4.addAfter(Construction5);
		Construction5.addAfter(Construction6);
		Construction6.addAfter(Construction7);
		Construction7.addAfter(Construction8);
		Construction8.addAfter(Construction9);
		Construction9.addAfter(Construction10);
		Construction10.addAfter(Construction1);

		//we want a left movement to pint to settings from anywhere
		Construction1.addLeft(MainMenu3);
		Construction2.addLeft(MainMenu3);
		Construction3.addLeft(MainMenu3);
		Construction4.addLeft(MainMenu3);
		Construction5.addLeft(MainMenu3);
		Construction6.addLeft(MainMenu3);
		Construction7.addLeft(MainMenu3);
		Construction8.addLeft(MainMenu3);
		Construction9.addLeft(MainMenu3);
		Construction10.addLeft(MainMenu3);

	MainMenu3.addAfter(MainMenu4);

	MainMenu4.addRight(Trailer1);
		//we want looping both up and down
		Trailer1.addBefore(Trailer10);
		Trailer1.addAfter(Trailer2);
		Trailer2.addAfter(Trailer3);
		Trailer3.addAfter(Trailer4);
		Trailer4.addAfter(Trailer5);
		Trailer5.addAfter(Trailer6);
		Trailer6.addAfter(Trailer7);
		Trailer7.addAfter(Trailer8);
		Trailer8.addAfter(Trailer9);
		Trailer9.addAfter(Trailer10);
		Trailer10.addAfter(Trailer1);

		//we want a left movement to pint to settings from anywhere
		Trailer1.addLeft(MainMenu4);
		Trailer2.addLeft(MainMenu4);
		Trailer3.addLeft(MainMenu4);
		Trailer4.addLeft(MainMenu4);
		Trailer5.addLeft(MainMenu4);
		Trailer6.addLeft(MainMenu4);
		Trailer7.addLeft(MainMenu4);
		Trailer8.addLeft(MainMenu4);
		Trailer9.addLeft(MainMenu4);
		Trailer10.addLeft(MainMenu4);

	MainMenu4.addAfter(MainMenu1);

	Check_Joys();
	command[6]=125;
	command[8]=125;
	Check_Encoders();
	Menu_Change = 1;
	menu.moveDown();
}

void RCTractorControllerV1::Update(){

    Menu_Update();

}

void RCTractorControllerV1::Check_Joys(){
	// Check Joy Stick Potentiometers
	_Left_Joy_X = analogRead(_JX_L);
	_Left_Joy_X= map(_Left_Joy_X, 0, 1024, 0, 255);
	_Left_Joy_Y = analogRead(_JY_L);
	_Left_Joy_Y= map(_Left_Joy_Y, 0, 1024, 0, 255);
	_Right_Joy_X = analogRead(_JX_R);
	_Right_Joy_X= map(_Right_Joy_X, 0, 1024, 0, 255);
	_Right_Joy_Y = analogRead(_JY_R);
	_Right_Joy_Y= map(_Right_Joy_Y, 0, 1024, 0, 255);

	if(_Left_Joy_X != _Left_Joy_X_Old || _Left_Joy_Y != _Left_Joy_Y_Old || _Right_Joy_X != _Right_Joy_X_Old || _Right_Joy_Y != _Right_Joy_Y_Old ){

		Throttle_Position = _Left_Joy_Y;

		if(Vehicle_Gear == 1){
			_Left_Joy_Y = map(Throttle_Position, 0, 255, 84, 171);
		}
		else if(Vehicle_Gear == 2){
			_Left_Joy_Y = map(Throttle_Position, 0, 255, 42, 213);
		}

		command[1]=_Left_Joy_X;
		command[2]=_Left_Joy_Y;
		command[3]=_Right_Joy_X;
		command[4]=_Right_Joy_Y;

		_Left_Joy_X_Old = _Left_Joy_X;
		_Left_Joy_Y_Old = _Left_Joy_Y;
		_Right_Joy_X_Old = _Right_Joy_X;
		_Right_Joy_Y_Old = _Right_Joy_Y;
	}
}


void RCTractorControllerV1::Check_Encoders(){
	// Check Joy Stick Potentiometers
	_POT1_Val = analogRead(_POT1);
	_POT1_Val = map(_POT1_Val, 0, 1024, 0, 255);
	_POT2_Val = analogRead(_POT2);
	_POT2_Val = map(_POT2_Val, 0, 1024, 0, 255);
	_POT3_Val = analogRead(_POT3);
	_POT3_Val = map(_POT3_Val, 0, 1024, 0, 255);
	_POT4_Val = analogRead(_POT4);
	_POT4_Val = map(_POT4_Val, 0, 1024, 0, 255);

	if(_POT1_Val != _POT1_Val_Old || _POT2_Val != _POT2_Val_Old || _POT3_Val != _POT3_Val_Old || _POT4_Val != _POT4_Val_Old ){

		command[5]=_POT1_Val;
		command[6]=_POT2_Val;
		command[7]=_POT3_Val;
		command[8]=_POT4_Val;

		_POT1_Val_Old = _POT1_Val;
		_POT2_Val_Old = _POT2_Val;
		_POT3_Val_Old = _POT3_Val;
		_POT4_Val_Old = _POT4_Val;
	}
}

void RCTractorControllerV1::Check_Light_Buttons(){
	// Check Joystick Switches
	if (digitalRead(_S_Left_Joy) == LOW || digitalRead(_S_Right_Joy) == LOW)
	{
		delay(10); // Allow time for both switches to activate
		if (digitalRead(_S_Left_Joy) == LOW && digitalRead(_S_Right_Joy) == LOW)
		{
			if((bitRead(_LED_Data, 0))!=1 && (bitRead(_LED_Data, 1))!=1){
				bitWrite(_LED_Data, 0, 1);  // Turn on Hazard Lights
				bitWrite(_LED_Data, 1, 1);
			}
			else{
				bitWrite(_LED_Data, 0, 0);  // Turn off Indicators
				bitWrite(_LED_Data, 1, 0);
			}

			// Wait for switch to be released
			while (digitalRead(_S_Left_Joy) == LOW || digitalRead(_S_Right_Joy) == LOW){}
		}
		else if (digitalRead(_S_Right_Joy) == LOW)
		{
			if ((bitRead(_LED_Data, 0))!=1){
				bitWrite(_LED_Data, 0, 1);  // Command to turn on right indicator
			}
			else {
				bitWrite(_LED_Data, 0, 0);  // Turn off Indicators
				bitWrite(_LED_Data, 1, 0);
			}

			// Wait for switch to be released
			while (digitalRead(_S_Right_Joy) == LOW){}
		}
		else if (digitalRead(_S_Left_Joy) == LOW)
		{
			if ((bitRead(_LED_Data, 1))!=1){
				bitWrite(_LED_Data, 1, 1);  // Command to turn on right indicator
			}
			else {
				bitWrite(_LED_Data, 0, 0);  // Turn off Indicators
				bitWrite(_LED_Data, 1, 0);
			}

			// Wait for switch to be released
			while (digitalRead(_S_Left_Joy) == LOW){}
		}
	}

	if ((digitalRead(_S_DB))==LOW)
	{
		if ((bitRead(_LED_Data, 2))!=1){
			bitWrite(_LED_Data, 2, 1);  // Turn on Dips
			bitWrite(_LED_Data, 3, 0);  // Turn off Full Beams
			bitWrite(_LED_Data, 4, 0);  // Turn off Work Lights
		}
		else {
			bitWrite(_LED_Data, 2, 0);  // Turn off Dips
			bitWrite(_LED_Data, 3, 0);  // Turn off Full Beams
			bitWrite(_LED_Data, 4, 0);  // Turn off Work Lights
		}

		// Wait for switch to be released
		while ((digitalRead(_S_DB)) == LOW){}
	}

	if (digitalRead(_S_FB) == LOW && bitRead(_LED_Data, 2)!=0)
	{
		if ((bitRead(_LED_Data, 3))!=1){
			bitWrite(_LED_Data, 2, 1);  // Turn on Dips
			bitWrite(_LED_Data, 3, 1);  // Turn on Full Beams
			bitWrite(_LED_Data, 4, 0);  // Turn off Work Lights
		}
		else {
			bitWrite(_LED_Data, 2, 1);  // Turn on Dips
			bitWrite(_LED_Data, 3, 0);  // Turn off Full Beams
			bitWrite(_LED_Data, 4, 0);  // Turn off Work Lights
		}

		// Wait for switch to be released
		while ((digitalRead(_S_FB)) == LOW){}
	}

	if ((digitalRead(_S_WL))==LOW && bitRead(_LED_Data, 2)!=0)
	{
		if ((bitRead(_LED_Data, 4))!=1){
			bitWrite(_LED_Data, 2, 1);  // Turn on Dips
			bitWrite(_LED_Data, 3, 1);  // Turn on Full Beams
			bitWrite(_LED_Data, 4, 1);  // Turn on Work Lights
		}
		else {
			bitWrite(_LED_Data, 2, 1);  // Turn on Dips
			bitWrite(_LED_Data, 3, 1);  // Turn on Full Beams
			bitWrite(_LED_Data, 4, 0);  // Turn off Work Lights
		}

		// Wait for switch to be released
		while ((digitalRead(_S_WL)) == LOW){}
	}
	command[9] = _LED_Data;
}


void RCTractorControllerV1::Check_Func_Buttons(){

	if ((digitalRead(_S_F1))==LOW)
	{
    	if ((bitRead(_FUNC_Data, 0))!=1){
      		bitWrite(_FUNC_Data, 0, 1);  // Turn on Function 1
    	}
    	else {
      		bitWrite(_FUNC_Data, 0, 0);  // Turn off Function 1
    	}

    	// Wait for switch to be released
    	while ((digitalRead(_S_F1)) == LOW){}
	}


	if ((digitalRead(_S_F2))==LOW)
  	{

    	if ((bitRead(_FUNC_Data, 1))!=1){
      		bitWrite(_FUNC_Data, 1, 1);  // Turn on Function 1
    	}
    	else {
      		bitWrite(_FUNC_Data, 1, 0);  // Turn off Function 1
    	}

    	// Wait for switch to be released
    	while ((digitalRead(_S_F2)) == LOW){}
  	}

  	if ((digitalRead(_S_F3))==LOW)
    {

		if ((bitRead(_FUNC_Data, 2))!=1){
        	bitWrite(_FUNC_Data, 2, 1);  // Turn on Function 1
      	}
      	else {
        	bitWrite(_FUNC_Data, 2, 0);  // Turn off Function 1
      	}

      	// Wait for switch to be released
      	while ((digitalRead(_S_F3)) == LOW){}
	}
	command[10] = _FUNC_Data;
}


void RCTractorControllerV1::Menu_Update(){
	if ((digitalRead(_S_Up))==LOW)
	{
		menu.moveUp();

		// Wait for switch to be released
		while ((digitalRead(_S_Up)) == LOW){}
	}

	if ((digitalRead(_S_Down))==LOW)
	{
		menu.moveDown();

		// Wait for switch to be released
		while ((digitalRead(_S_Down)) == LOW){}
	}

	if ((digitalRead(_S_Back))==LOW)
	{
		if(Menu_Level == 1){
			menu.moveLeft();
			Menu_Change = 1;
			menu.moveDown();
			menu.moveUp();
		}

		// Wait for switch to be released
		while ((digitalRead(_S_Back)) == LOW){}
	}

	if ((digitalRead(_S_Select))==LOW)
	{
		if(Menu_Level == 0){
			menu.moveRight();
			Menu_Change = 1;
			menu.moveDown();
		}
		else if(Menu_Level == 1){
			menu.use();
		}

		// Wait for switch to be released
		while ((digitalRead(_S_Select)) == LOW){}
	}
}

void RCTractorControllerV1::Check_Fast_Select(){

	if ((digitalRead(_VQS_V1))==LOW)
  {
	  _V_ID = 0;
	  // Wait for switch to be released
	      while ((digitalRead(_VQS_V1)) == LOW)
	      {

    }
  }

	else if ((digitalRead(_VQS_V2))==LOW)
  {
	  _V_ID = 1;
	  // Wait for switch to be released
	      while ((digitalRead(_VQS_V2)) == LOW)
	      {

    }
  }

  	else if ((digitalRead(_VQS_V3))==LOW)
    {
		_V_ID = 2;
  	  // Wait for switch to be released
  	      while ((digitalRead(_VQS_V3)) == LOW)
  	      {

      }
  }

  	else if ((digitalRead(_VQS_V4))==LOW)
    {
		_V_ID = 3;
  	  // Wait for switch to be released
  	      while ((digitalRead(_VQS_V4)) == LOW)
  	      {

      }
  }

  	else if ((digitalRead(_VQS_V5))==LOW)
    {
		_V_ID = 4;
  	  // Wait for switch to be released
  	      while ((digitalRead(_VQS_V5)) == LOW)
  	      {

      }
  }

  	else if ((digitalRead(_VQS_V6))==LOW)
    {
		_V_ID = 5;
  	  // Wait for switch to be released
  	      while ((digitalRead(_VQS_V6)) == LOW)
  	      {

      }
  }

    	else if ((digitalRead(_VQS_V7))==LOW)
      {
  		_V_ID = 6;
    	  // Wait for switch to be released
    	      while ((digitalRead(_VQS_V7)) == LOW)
    	      {

        }
    }

    	else if ((digitalRead(_VQS_V8))==LOW)
      {
  		_V_ID = 7;
    	  // Wait for switch to be released
    	      while ((digitalRead(_VQS_V8)) == LOW)
    	      {

        }
    }

    	else if ((digitalRead(_VQS_V9))==LOW)
      {
  		_V_ID = 8;
    	  // Wait for switch to be released
    	      while ((digitalRead(_VQS_V9)) == LOW)
    	      {

        }
    }

    	else if ((digitalRead(_VQS_V10))==LOW)
      {
  		_V_ID = 9;
    	  // Wait for switch to be released
    	      while ((digitalRead(_VQS_V10)) == LOW)
    	      {

        }
  }

//command[0] = _V_ID;

}

/*******************************************************************/
/*       Functions to Test Controller and Vehicle Voltages         */
/*******************************************************************/

long RCTractorControllerV1::readVcc() {
  long result;
  // Read 1.1V reference against AVcc
  ADMUX = _BV(REFS0) | _BV(MUX3) | _BV(MUX2) | _BV(MUX1);ADMUX = _BV(REFS0) | _BV(MUX4) | _BV(MUX3) | _BV(MUX2) | _BV(MUX1); ADCSRB &= ~_BV(MUX5);
  delay(2); // Wait for Vref to settle
  ADCSRA |= _BV(ADSC); // Convert
  while (bit_is_set(ADCSRA,ADSC));
  result = ADCL;
  result |= ADCH<<8;
  result = (1126400L / result); // Back-calculate AVcc in mV
  return result;
}

void RCTractorControllerV1::Controller_Voltage()
{
	if(Controller_V_Measure_Period==50){
    	if(Controller_V_Readings[(numReadings-1)]==0){
			Controller_V_Measure_Period=49;
		}
		else{
    		Controller_V_Measure_Period=0;
		}

		// subtract the last reading:
		Controller_V_Total = Controller_V_Total - Controller_V_Readings[Controller_V_ReadIndex];
		// read from the sensor:
		Controller_V_Readings[Controller_V_ReadIndex] = analogRead(A5);
		// add the reading to the total:
		Controller_V_Total = Controller_V_Total + Controller_V_Readings[Controller_V_ReadIndex];
		// advance to the next position in the array:
		Controller_V_ReadIndex = Controller_V_ReadIndex + 1;

		// if we're at the end of the array...
		if (Controller_V_ReadIndex >= numReadings) {
			// ...wrap around to the beginning:
			Controller_V_ReadIndex = 0;
		}

		// calculate the average:
		Controller_V_Average = Controller_V_Total / numReadings;

		Controller_V_Average = (Controller_V_Average / 1024) * (readVcc());

		Controller_V_Average = (Controller_V_Average / Denominator)/1000;
	}
	Controller_V_Measure_Period = Controller_V_Measure_Period+1;
}


void RCTractorControllerV1::Vehicle_Voltage(long _Vehicle_V){
	Vehicle_V = _Vehicle_V;

	if(Vehicle_V_Measure_Period==25){
		if(Vehicle_V_Readings[(numReadings-1)]==0){
			Vehicle_V_Measure_Period=24;
		}
		else{
			Vehicle_V_Measure_Period=0;
		}

		Vehicle_V_Measure_Period=0;

		// subtract the last reading:
		Vehicle_V_Total = Vehicle_V_Total - Vehicle_V_Readings[Vehicle_V_ReadIndex];
		// read from the sensor:
		Vehicle_Volt =Vehicle_V;
		Vehicle_V_Readings[Vehicle_V_ReadIndex] = Vehicle_Volt/1000;
		// add the reading to the total:
		Vehicle_V_Total = Vehicle_V_Total + Vehicle_V_Readings[Vehicle_V_ReadIndex];
		// advance to the next position in the array:
		Vehicle_V_ReadIndex = Vehicle_V_ReadIndex + 1;

		// if we're at the end of the array...
		if (Vehicle_V_ReadIndex >= numReadings) {
			// ...wrap around to the beginning:
			Vehicle_V_ReadIndex = 0;
		}

		// calculate the average:
		Vehicle_V_Average = Vehicle_V_Total / numReadings;
	}
	Vehicle_V_Measure_Period = Vehicle_V_Measure_Period+1;
}

/*******************************************************************/
/*                        Menu Backgrounds                         */
/*******************************************************************/


/*******************************************************************/
/*           Functions used to Control RC Vehicles                 */
/*******************************************************************/

void RCTractorControllerV1::Start_Hitachi(){
	int arm1_pos = 127;
	int arm2_pos = 127;
	int slew_pos = 127;

	int arm1_timer = millis();
	int arm1_space = 1000;
	int arm1_dir = 0;

	while(Vehicle_Active != 0){
   		Check_Joys();
		Check_Encoders();

		if(command[3] < 125){
			arm1_space = map(command[3],125,0,1000,20);
    		arm1_dir = 1;
		}
		else if(command[3]>130){
			arm1_space = map(command[3],130,255,1000,20);
    		arm1_dir = 2;
		}
		else{arm1_dir = 0;}

		if((millis()-arm1_timer)>arm1_space){
		    if (arm1_pos != 0x00 && arm1_dir == 1){
		      arm1_pos --;
		        delay(5);
		      Serial.write(0x01);  // Command to enter steering mode
		      delay(5);
		      Serial.write(arm1_pos);  // Steering value
		    }
		    else if (arm1_pos != 0xFF && arm1_dir == 2){
		      arm1_pos ++;
		         delay(5);
		      Serial.write(0x01);  // Command to enter steering mode
		      delay(5);
		      Serial.write(arm1_pos);  // Steering value
		    }
		    arm1_timer = millis();
	   }

/*
		  if(command[2] < 100){
		    if (arm2_pos != 0x00){
		      arm2_pos = arm2_pos - 3;
		       delay(5);
		      Serial.write(0x02);  // Command to enter drive mode
		      delay(5);
		      Serial.write(arm2_pos);  // Drive value
		    }
		  }
		  else if(command[2]>150){
		    if (arm2_pos != 0xFF){
		      arm2_pos = arm2_pos + 3;
		    delay(5);
		      Serial.write(0x02);  // Command to enter drive mode
		      delay(5);
		      Serial.write(arm2_pos);  // Drive value
		    }
		  }


		       delay(5);
		      Serial.write(0x03);  // Command to enter drive mode
		      delay(5);
		      Serial.write(command[4]);  // Drive value



		  if(command[1] < 100){
		    if (slew_pos != 0x00){
		      slew_pos = slew_pos -2;
		      delay(5);
		      Serial.write(0x04);  // Command to enter drive mode
		      delay(5);
		      Serial.write(slew_pos);  // Drive value
		    }
		  }
		  else if(command[1]>150){
		    if (slew_pos != 0xFF){
		      slew_pos = slew_pos +2;
		    delay(5);
		      Serial.write(0x04);  // Command to enter drive mode
		      delay(5);
		      Serial.write(slew_pos);  // Drive value
		    }
		  }


		    delay(5);
		  Serial.write(0x06);  // Command to enter drive mode
		  delay(5);
		  Serial.write(command[5]);  // Drive value

		    delay(5);
		  Serial.write(0x05);  // Command to enter drive mode
		  delay(5);
  Serial.write(command[7]);  // Drive value

		if ((digitalRead(_S_Back))==LOW)
		{
			Vehicle_Active = 0;
			menu.moveUp();
			Menu_Change = 1;
			menu.moveDown();

			// Wait for switch to be released
			while ((digitalRead(_S_Back)) == LOW){}
    	}*/
	}
}

void RCTractorControllerV1::Start_Vehicle(){

	command[0]=Vehicle_Active;
	Vehicle_Gear = 3;
	Lost_Packet_Count = 0;
	Reception_Meter = 0;
	Battery_Meter = 0;
	if(Com_Method == 0){}
	if(Com_Method == 1){}
	if(Com_Method == 2){}
	Controller_V_Measure_Period=50;
	Vehicle_V_Measure_Period=25;
	unsigned long time = micros();
	Send_Timer = millis();

	while(Vehicle_Active != 0){

		if((micros() - time)>500000){
			time = micros();
		}

		//Check_Fast_Select();
		Check_Light_Buttons();
		Check_Func_Buttons();

		Controller_Voltage();

		// Tractor_Voltage();

		Update_Dash(0);

   		Check_Joys();
		Check_Encoders();

  		Vehicle_Voltage(Vehicle_V);

		Update_Dash(1);

		if ((digitalRead(_S_Up))==LOW)
		{
			if(Vehicle_Gear != 3){

				Vehicle_Gear++;

				if(Vehicle_Gear == 3){

				}
				else if(Vehicle_Gear == 2){

				}

			}

			// Wait for switch to be released
			while ((digitalRead(_S_Up)) == LOW){}
    	}

		if ((digitalRead(_S_Down))==LOW)
		{
			if(Vehicle_Gear != 1){

				Vehicle_Gear--;

				if(Vehicle_Gear == 2){

				}
				else if(Vehicle_Gear == 1){

				}
			}

			// Wait for switch to be released
			while ((digitalRead(_S_Down)) == LOW){}
    	}

		if(Com_Method == 0){Send_NRF();}
		if(Com_Method == 1){Send_NRF();}

		if ((digitalRead(_S_Back))==LOW)
		{
			Vehicle_Active = 0;
			menu.moveUp();
			Menu_Change = 1;
			menu.moveDown();

			// Wait for switch to be released
			while ((digitalRead(_S_Back)) == LOW){}
    	}
	}
}

void RCTractorControllerV1::Send_NRF(){
  		radio.write( command, sizeof(command));

  		if ( radio.isAckPayloadAvailable() )
  		{
    		radio.read(&Vehicle_V, sizeof(Vehicle_V));
    		Lost_Packet_Count = 0;
  		}
  		else {
    		Lost_Packet_Count++;
  		}
}

void RCTractorControllerV1::Send_XBee(){

}


void RCTractorControllerV1::Update_Dash(int clear){

}



unsigned char RCTractorControllerV1::bitswap (unsigned char x)
{
 byte result;

   asm("mov __tmp_reg__, %[in] \n\t"
     "lsl __tmp_reg__  \n\t"   /* shift out high bit to carry */
     "ror %[out] \n\t"  /* rotate carry __tmp_reg__to low bit (eventually) */
     "lsl __tmp_reg__  \n\t"   /* 2 */
     "ror %[out] \n\t"
     "lsl __tmp_reg__  \n\t"   /* 3 */
     "ror %[out] \n\t"
     "lsl __tmp_reg__  \n\t"   /* 4 */
     "ror %[out] \n\t"

     "lsl __tmp_reg__  \n\t"   /* 5 */
     "ror %[out] \n\t"
     "lsl __tmp_reg__  \n\t"   /* 6 */
     "ror %[out] \n\t"
     "lsl __tmp_reg__  \n\t"   /* 7 */
     "ror %[out] \n\t"
     "lsl __tmp_reg__  \n\t"   /* 8 */
     "ror %[out] \n\t"
     : [out] "=r" (result) : [in] "r" (x));
     return(result);
}

void RCTractorControllerV1::Control32_Pulse(int Num_Pulse){
	cli();
	while(Num_Pulse!=0){
		PINL = 0x04; // Toggle pin47
		__asm__("nop\n\tnop\n\tnop\n\tnop\n\tnop\n\t");
		__asm__("nop\n\tnop\n\tnop\n\tnop\n\tnop\n\t");
		__asm__("nop\n\tnop\n\tnop\n\tnop\n\tnop\n\t");
		PINL = 0x04; // Toggle pin 47
		__asm__("nop\n\tnop\n\tnop\n\tnop\n\tnop\n\t");
		__asm__("nop\n\tnop\n\t");
  		__asm__("nop\n\t");
		Num_Pulse --;
	}
	sei();
	delayMicroseconds(80);
}



